# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - IncludeComments Macro

    @copyright: 2013 by Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL (v2 or later), see COPYING.txt for details.

    Code from the Include macro:

    @copyright: 2000-2004 Juergen Hermann <jh@web.de>,
                2000-2001 Richard Jones <richard@bizarsoftware.com.au>
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

from MoinMoin.Page import Page
from MoinMoin.macro import Include
from MoinMoin.user import User
from MoinMoin.wikiutil import escape
from SubpageComments import get_comment_numbers
import re
import codecs

try:
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO

__version__ = "0.1"

Dependencies = ['pages']

# Macro functions.

def execute(macro, text):
    request = macro.request
    fmt = request.formatter
    page = request.page
    pagename = page.page_name
    _ = request.getText

    output = []
    append = output.append

    # Add a heading.

    append(fmt.heading(on=1, depth=1))
    append(fmt.text(_("Comments")))
    append(fmt.heading(on=0, depth=1))

    # Provide a form for adding new comments.

    if request.user.valid and request.user.may.write(pagename):

        d = {
            "show_form"         : escape(_("Add a comment to this page.")),
            "comment_label"     : escape(_("Write a comment in the box.")),
            "comment_default"   : "",
            "submit"            : escape(_("Submit this comment")),
            }

        append("""\
<div id="includecomments-anchor">
<a href="#includecomments-anchor">%(show_form)s</a>
<form action="?action=PostComment" method="post" class="includecomments-form">
<p>%(comment_label)s</p>
<textarea name="comment" cols="60" rows="10">%(comment_default)s</textarea>
<p><input name="submit" type="submit" value="%(submit)s" /></p>
</form>
</div>
""" % d)

    # Add included comments.

    pages = get_comment_numbers(pagename, request)

    ownerfn = re.compile("^#pragma comment-owner (.*?)$", re.MULTILINE | re.UNICODE).search

    # NOTE: Much of the code below originates from the Include macro, but it
    # NOTE: excludes various options of that macro and adds comment-related
    # NOTE: output.

    # Track included pages.

    if not hasattr(page, '_macroInclude_pagelist'):
        page._macroInclude_pagelist = {}

    # Visit each comment page.

    for inc_name in pages:
        inc_name = "%s/%04d" % (pagename, inc_name)

        # Skip unreadable or already included pages.

        if not request.user.may.read(inc_name):
            continue
        if inc_name in page._macroInclude_pagelist:
            continue

        # Obtain a separate formatter for the included page.

        inc_fmt = macro.formatter.__class__(request, is_included=True)
        inc_fmt._base_depth = macro.formatter._base_depth

        # Obtain the included page.

        inc_page = Page(request, inc_name, formatter=inc_fmt)
        if not inc_page.exists():
            continue
        inc_page._macroInclude_pagelist = page._macroInclude_pagelist

        if not hasattr(request, "_Include_backto"):
            request._Include_backto = pagename

        # Output a container for the included page.

        append(fmt.div(1, id=inc_name, css_class="included-comment"))

        # Add a label indicating a comment.

        match = ownerfn(inc_page.get_raw_body())
        if match:
            user = User(request, auth_username=match.group(1))
            if user.exists():
                append(fmt.div(1, css_class="included-comment-owner"))
                append(fmt.text(user.aliasname or user.name))
                append(fmt.div(0))

        # Set or increment include marker.

        page._macroInclude_pagelist[inc_name] = \
            page._macroInclude_pagelist.get(inc_name, 0) + 1

        # Output the included page.

        strfile = codecs.getwriter("utf-8")(StringIO())
        request.redirect(strfile)
        try:
            inc_page.send_page(content_only=True,
                               omit_footnotes=True,
                               count_hit=False)
            append(unicode(strfile.getvalue(), "utf-8"))
        finally:
            request.redirect()

        # Decrement or remove include marker.

        if page._macroInclude_pagelist[inc_name] > 1:
            page._macroInclude_pagelist[inc_name] = \
                page._macroInclude_pagelist[inc_name] - 1
        else:
            del page._macroInclude_pagelist[inc_name]

        # Close the container for the included page.

        append(fmt.div(0))

    return u"".join(output)

# vim: tabstop=4 expandtab shiftwidth=4
